package server

import (
	"fmt"
	"net/http"
	"time"

	"git.javil.eu/jacob1123/budgeteer/postgres"
	"github.com/gin-gonic/gin"
	"github.com/google/uuid"
)

type NewTransactionPayload struct {
	Date  JSONDate `json:"date"`
	Payee struct {
		ID   uuid.NullUUID
		Name string
	} `json:"payee"`
	Category struct {
		ID   uuid.NullUUID
		Name string
	} `json:"category"`
	Memo      string    `json:"memo"`
	Amount    string    `json:"amount"`
	BudgetID  uuid.UUID `json:"budgetId"`
	AccountID uuid.UUID `json:"accountId"`
	State     string    `json:"state"`
}

func (h *Handler) newTransaction(c *gin.Context) {
	var payload NewTransactionPayload
	err := c.BindJSON(&payload)
	if err != nil {
		c.AbortWithError(http.StatusBadRequest, err)
		return
	}

	amount := postgres.Numeric{}
	err = amount.Set(payload.Amount)
	if err != nil {
		c.AbortWithError(http.StatusBadRequest, fmt.Errorf("amount: %w", err))
		return
	}

	payeeID := payload.Payee.ID
	if !payeeID.Valid && payload.Payee.Name != "" {
		newPayee := postgres.CreatePayeeParams{
			Name:     payload.Payee.Name,
			BudgetID: payload.BudgetID,
		}
		payee, err := h.Service.CreatePayee(c.Request.Context(), newPayee)
		if err != nil {
			c.AbortWithError(http.StatusInternalServerError, fmt.Errorf("create payee: %w", err))
		}

		payeeID = uuid.NullUUID{
			UUID:  payee.ID,
			Valid: true,
		}
	}

	newTransaction := postgres.CreateTransactionParams{
		Memo:       payload.Memo,
		Date:       time.Time(payload.Date),
		Amount:     amount,
		AccountID:  payload.AccountID,
		PayeeID:    payeeID,
		CategoryID: payload.Category.ID,
		Status:     postgres.TransactionStatus(payload.State),
	}
	transaction, err := h.Service.CreateTransaction(c.Request.Context(), newTransaction)
	if err != nil {
		c.AbortWithError(http.StatusInternalServerError, fmt.Errorf("create transaction: %w", err))
		return
	}

	c.JSON(http.StatusOK, transaction)
}
